﻿//Example program on how to use Rohde & Schwarz IVI.NET RsSpecan driver
//Read more on Rohde & Schwarz IVI.NET here: 1MA268 How to use Rohde & Schwarz IVI.NET instrument drivers
//Before running the application change the Resource string to pass your device. E.G. "TCPIP::192.168.1.101::HISLIP"
//Application note link http://www.rohde-schwarz.com/appnote/1MA268
//Author: Miloslav Macko

// Version 2.0:
//- Adjusted for the RsSpecAn driver version 1.6.0.0
//- Changed Exceptions handling to Ivi.Driver exceptions.
//- removed the use of extention function driver.Extended_WAI(), because
//   the driver already has the method integrated in: driver.UtilityFunctions.ProcessAllPreviousCommands();

// Version 1.0:
//- Initial version

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Globalization;

using System.Runtime.InteropServices;
using System.IO;

using Ivi.Driver;
using Ivi.SpecAn;
using RohdeSchwarz.RsSpecAn; //driver namespace
using RsSpecanExtentions; //driver extention functions

namespace RsSpecan_IVI.NET_Complex_Example
{
    class Program
    {
        static void Main(string[] args)
        {
            //set the regional settings to fixed: decimal point will always be '.'
            Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
            string exceptionMessage = null;
            RsSpecAn driver = null;
            try
            {
                #region Initializing of a HISLIP session
                driver = new RsSpecAn("TCPIP::192.168.1.101::HISLIP", true, true, "Simulate=False");
                
                var sTimeInvariant = driver.SweepCoupling.SweepTime;

                //the driver checks the range of the entered parameters. Default value: true
                driver.DriverOperation.RangeCheck = true;

                //query the device status "*STB?" after every command. Default value: true
                driver.DriverOperation.QueryInstrumentStatus = true;

                //check if the property/function is available for the instrument. Default value: true
                driver.UtilityFunctions.OptionChecking = true;

                driver.Display.Update = true;
                #endregion

                #region Read basic info
                if (!driver.DriverOperation.Simulate)
                {
                    Console.WriteLine("Application versions: {0}\n\n\n", driver.System.AppVersions.Replace(",", "\n"));
                }
                else
                {
                    Console.WriteLine("Driver session is in simulation mode");
                }

                driver.System.ClearErrors();
                #endregion

                #region Basic settings
                double centerFrequency = 1.5E9;
                double spanFrequency = 200E6;
                int sweepCount = 10;

                driver.DriverOperation.QueryInstrumentStatus = false;
                driver.Frequency.CenterFrequency = centerFrequency;
                driver.System.ClearErrors();
                driver.Frequency.Span = spanFrequency;
                driver.DriverOperation.QueryInstrumentStatus = true;

                driver.Acquisition.SweepMode = false;
                driver.Acquisition.NumberOfSweeps = sweepCount;

                driver.Amplitude.ReferenceLevel = 20;
                driver.Display.Range = 160;
                driver.SweepCoupling.ResolutionBandwidth = 2e3;
                driver.SweepCoupling.VideoBandwidthAuto = true;
                #endregion

                #region Trace1 settings
                var Trace1 = driver.Trace["TR1"];
                var Trace1_IVI = driver.Traces["TR1"];

                var Trace1_W0 = Trace1.Window["Win1"];

                Trace1_W0.Enabled = true;
                Trace1_W0.Type = TraceMode.Maxhold;
                #endregion
                
                #region Trace2 settings
                var Trace2 = driver.Trace["TR2"];
                var Trace2_IVI = driver.Traces["TR2"];
                var Trace2_W0 = Trace2.Window["Win1"];
                Trace2_W0.Enabled = true;
                Trace2_W0.Type = TraceMode.Minhold;
                #endregion

                //Acquisition;
                #region Acquisition
                Console.Write("\nAcquisition Started...");
                driver.Measurement.InitiateWithoutOPC();
                driver.Extended_WaitForMeasuringEnd (100);
                Console.WriteLine("Finished");
                #endregion

                double[] traceXdata = Trace1.FetchX(1);
                double[] traceYdata = Trace1.FetchY(1, TraceResultType.Trace1);

                //marker setting and reading
                var Marker = driver.Marker["M1"];
                var position = Marker.Position;
                                
                //delta marker
                var dMarker = Marker.Delta;
                dMarker.Enabled = true;
                dMarker.Mode = AbsoluteRelative.Relative;
                dMarker.Search.Minimum();

                #region Reading traces to ISpectrum structure
                //Ispectrum - cannot be used for ZeroSpan
                Console.Write("Reading traces...");
                ISpectrum<double> spectrumTrace1 = null;
                ISpectrum<double> spectrumTrace2 = null;

                spectrumTrace1 = Trace1_IVI.FetchY(spectrumTrace1);
                spectrumTrace2 = Trace2_IVI.FetchY(spectrumTrace2);
                Console.WriteLine("Finished");
                #endregion

                #region Hardcopy
                Console.Write("Hardcopy...");
                string filePathFSx = @"C:\Temp\IVI.NET_screenshot_FSx.png";
                string filePathPC = @"C:\IVI.NET_screenshot_PC.png";
                driver.HardCopy.CommentScreen = "Taken by IVI.NET driver RC application";
                driver.HardCopy.FileName = filePathFSx;

                driver.HardCopy.Color.Color = true;
                driver.HardCopy.Color.HcopyColorPreset(4);

                var Hcpy = driver.HardCopy.Destination["DE1"];
                Hcpy.DeviceDestination = HcopyDevice.File;
                
                Hcpy.LanguageFormat = HcopyDeviceLang.PNG;
                Hcpy.TimeDate = true;
                Hcpy.Print();
                //transfer of the file to the PC
                driver.UtilityFunctions.ReadToFileFromInstrument(filePathFSx, filePathPC);
                Console.WriteLine("Finished, screenshot file saved to '{0}'", filePathPC);
                #endregion

                //starting the Windows Form with the traces and screenshot
                System.Windows.Forms.Application.Run(new SpectrumChart(spectrumTrace1, spectrumTrace2, filePathPC));
            }
            #region Exceptions handling

            //driver IO error
            catch (Ivi.Driver.IOException exception)
            {
                exceptionMessage = "\nIOException:\n"
                                    + exception.Message
                                    + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //driver function is not supported by this sensor
            catch (Ivi.Driver.OperationNotSupportedException exception)
            {
                exceptionMessage = "\nOperationNotSupportedException:\n"
                                    + exception.Message
                                    + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //when a task takes longer than the defined OPC timeout
            catch (Ivi.Driver.MaxTimeExceededException exception)
            {
                exceptionMessage = "\nMaxTimeExceededException:\n"
                                   + exception.Message
                                   + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //if the instrument returns an error in the error queue
            catch (Ivi.Driver.InstrumentStatusException exception)
            {
                exceptionMessage = "\nInstrumentStatusException:\n"
                                 + exception.Message
                                 + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //when the used waveform capacity is not enough
            catch (Ivi.Driver.DataArrayTooSmallException exception)
            {
                exceptionMessage = "\nDataArrayTooSmallException - Waveform array is too small for the data:\n"
                                 + exception.Message
                                 + "\n\nStackTrace\n" + exception.StackTrace;
            }

            //when entered property or method value is out of range
            catch (Ivi.Driver.OutOfRangeException exception)
            {
                exceptionMessage = "\nOutOfRangeException:\n"
                                    + exception.Message
                                    + "\n\nStackTrace\n" + exception.StackTrace;
            }
                        
            //----------------------------------------------------------------------------------------------------------------
            //finally section
            finally
            //this section is always performed regardless whether any exception was raised or not
            {
                //show exception message if it exists
                if (exceptionMessage != null)
                {
                    Console.WriteLine(exceptionMessage);
                }

                //close the driver and disposing of the driver object
                if (driver != null) driver.Close();
                driver.Dispose();

                Console.Out.WriteLine("\n\nPress any key to finish...");
                Console.ReadKey();
            }
            #endregion
            //----------------------------------------------------------------------------------------------------------------
        }
    }
}

//Example for user-defined extensions methods of the RsSpecAn object
namespace RsSpecanExtentions
{
    public static class Extensions_of_RsSpecan
    {
        //----------------------------------------------------------------------------------------------------------------
        #region Measuring?
        /// <summary>
        /// <para>Waiting for the measuremement to end with adaptable polling interval:</para>
        /// <para>The first 100ms the polling is performed with the 10ms pause.</para>
        /// <para>Up to 1s the polling is performed with the 50ms pause.</para>
        /// <para>Up to 5s the polling is performed with the 200ms pause.</para>
        /// <para>Above 5s the polling is performed with the 500ms pause.</para>
        /// </summary>
        /// <param name="driver"></param>
        /// <param name="timeoutMs"></param>
        public static void Extended_WaitForMeasuringEnd(this RsSpecAn driver, int timeoutMs)
        {
            var start = DateTime.Now;
            var stop = start.AddMilliseconds(timeoutMs);
            RohdeSchwarz.RsSpecAn.AcquisitionStatus acqStatus;
            bool complete = false;
            do
            {
                acqStatus = driver.Measurement.AcquisitionStatus();
                complete = (acqStatus == RohdeSchwarz.RsSpecAn.AcquisitionStatus.Complete);

                var interval = (int)DateTime.Now.Subtract(start).TotalMilliseconds;

                if (interval < 100) Thread.Sleep(10);
                else if (interval < 1000) Thread.Sleep(50);
                else if (interval < 5000) Thread.Sleep(200);
                else Thread.Sleep(500);

            } while (!complete && ((DateTime.Now) < stop));
            if (!complete)
                throw new Ivi.Driver.MaxTimeExceededException(String.Format("Extended_WaitForMeasuringEnd timeout (maximum time {0} ms)", timeoutMs));
        }
        #endregion
    }
}